#pragma once

#include <jenga/include/common/BoostXmlSupport.h>

#include <option.h>
#include <Program.h>
#include <Class.h>
#include <Procedure.h>
#include <TypeDef.h>
#include <Variable.h>
#include <Const.h>

class MetaImpl : public Jenga::Common::BoostXmlSupport<MetaImpl>
{
	// O
	NamespaceScopesCollection namespaceScopesCollection;

	// ֐E\bh
	UserProcs userProcs;

	// DLL֐
	DllProcs dllProcs;

	// NX
	Classes classesImpl;

	// O[oϐ
	Variables globalVars;

	// O[o萔
	Consts globalConsts;

	// O[o萔}N
	ConstMacros globalConstMacros;

	// blittable^
	BlittableTypes blittableTypes;

	// TypeDef
	TypeDefCollection typeDefs;

	// ֐|C^
	ProcPointers procPointers;

	// XMLVACYp
private:
	virtual const char *RootTagName() const
	{
		return "metaImpl";
	}
	friend class boost::serialization::access;
	template<class Archive> void serialize(Archive& ar, const unsigned int version)
	{
		trace_for_serialize( "serializing - MetaImpl" );

		ar & BOOST_SERIALIZATION_NVP( namespaceScopesCollection );
		ar & BOOST_SERIALIZATION_NVP( userProcs );
		ar & BOOST_SERIALIZATION_NVP( dllProcs );
		ar & BOOST_SERIALIZATION_NVP( classesImpl );
		ar & BOOST_SERIALIZATION_NVP( globalVars );
		ar & BOOST_SERIALIZATION_NVP( globalConsts );
		ar & BOOST_SERIALIZATION_NVP( globalConstMacros );
		ar & BOOST_SERIALIZATION_NVP( blittableTypes );
		ar & BOOST_SERIALIZATION_NVP( typeDefs );
		ar & BOOST_SERIALIZATION_NVP( procPointers );
	}

	Classes *pNowClassesForDebugger;

public:
	MetaImpl()
		: classesImpl()
		, pNowClassesForDebugger( &classesImpl )
	{
	}

	const NamespaceScopesCollection &GetNamespaces() const
	{
		return namespaceScopesCollection;
	}
	NamespaceScopesCollection &GetNamespaces()
	{
		return namespaceScopesCollection;
	}

	const UserProcs &GetUserProcs() const
	{
		return userProcs;
	}
	UserProcs &GetUserProcs()
	{
		return userProcs;
	}

	const DllProcs &GetDllProcs() const
	{
		return dllProcs;
	}
	DllProcs &GetDllProcs()
	{
		return dllProcs;
	}

	Classes &GetClasses()
	{
		return *pNowClassesForDebugger;
	}
	void SetClasses( Classes *pClasses )
	{
		this->pNowClassesForDebugger = pClasses;
	}

	const Variables &GetGlobalVars() const
	{
		return globalVars;
	}
	Variables &GetGlobalVars()
	{
		return globalVars;
	}

	const Consts &GetGlobalConsts() const
	{
		return globalConsts;
	}
	Consts &GetGlobalConsts()
	{
		return globalConsts;
	}

	const ConstMacros &GetGlobalConstMacros() const
	{
		return globalConstMacros;
	}
	ConstMacros &GetGlobalConstMacros()
	{
		return globalConstMacros;
	}

	BlittableTypes &GetBlittableTypes()
	{
		return blittableTypes;
	}

	TypeDefCollection &GetTypeDefs()
	{
		return typeDefs;
	}

	// ֐|C^
	ProcPointers &GetProcPointers()
	{
		return procPointers;
	}

	bool AutoWrite( const std::string &filePath )
	{
		std::ofstream ofs( filePath.c_str() );

		bool isSuccessful = false;
		try{
			boost::archive::xml_oarchive oa(ofs);

			// t@CǍ
			oa << boost::serialization::make_nvp( RootTagName(), *this );

			isSuccessful = true;
		}
		catch(...){
			// s
		}

		// ͂
		ofs.close();

		return isSuccessful;
	}
};
